window.Gleam.initSearch([{"doc":"gleam_pgo","title":"gleam_pgo","content":"# Gleam PGO\n\nA PostgreSQL database client for Gleam, based on [PGO][erlang-pgo].\n\n[erlang-pgo]: https://github.com/erleans/pgo\n\n```gleam\nimport gleam/pgo\nimport gleam/dynamic\nimport gleeunit/should\n\npub fn main() {\n  // Start a database connection pool.\n  // Typically you will want to create one pool for use in your program\n  let db = pgo.connect(pgo.Config(\n    ..pgo.default_config(),\n    host: &quot;localhost&quot;,\n    database: &quot;my_database&quot;,\n    pool_size: 15,\n  ))\n\n  // An SQL statement to run. It takes one int as a parameter\n  let sql = &quot;\n  select\n    name, age, colour, friends\n  from\n    cats\n  where\n    id = $1&quot;\n\n  // This is the decoder for the value returned by the query\n  let return_type = dynamic.tuple4(\n    dynamic.string,\n    dynamic.int,\n    dynamic.string,\n    dynamic.list(dynamic.string),\n  )\n\n  // Run the query against the PostgreSQL database\n  // The int `1` is given as a parameter\n  let assert Ok(response) = \n    pgo.execute(sql, db, [pgo.int(1)], return_type)\n\n  // And then do something with the returned results\n  response.count\n  |&gt; should.equal(2)\n  response.rows\n  |&gt; should.equal([\n    #(&quot;Nubi&quot;, 3, &quot;black&quot;, [&quot;Al&quot;, &quot;Cutlass&quot;]),\n  ])\n}\n```\n\n## Installation\n\n```sh\ngleam add gleam_pgo\n```\n\n## Atom generation\n\nCreating a connection pool with the `pgo.connect` function dynamically generates\nan Erlang atom. Atoms are not garbage collected and only a certain number of\nthem can exist in an Erlang VM instance, and hitting this limit will result in\nthe VM crashing. Due to this limitation you should not dynamically open new\nconnection pools, instead create the pools you need when your application starts\nand reuse them throughout the lifetime of your program.\n","url":"index.html"},{"doc":"gleam/pgo","title":"Config","content":"pub type Config {\n  Config(\n    host: String,\n    port: Int,\n    database: String,\n    user: String,\n    password: Option(String),\n    ssl: Bool,\n    connection_parameters: List(#(String, String)),\n    pool_size: Int,\n    queue_target: Int,\n    queue_interval: Int,\n    idle_interval: Int,\n    trace: Bool,\n    ip_version: IpVersion,\n  )\n}\n The configuration for a pool of connections.\nConfig(\n  host: String,\n  port: Int,\n  database: String,\n  user: String,\n  password: Option(String),\n  ssl: Bool,\n  connection_parameters: List(#(String, String)),\n  pool_size: Int,\n  queue_target: Int,\n  queue_interval: Int,\n  idle_interval: Int,\n  trace: Bool,\n  ip_version: IpVersion,\n)\n\nhost\n&lt;p&gt;(default: 127.0.0.1): Database server hostname.&lt;/p&gt;\n\nport\n&lt;p&gt;(default: 5432): Port the server is listening on.&lt;/p&gt;\n\ndatabase\n&lt;p&gt;Name of database to use.&lt;/p&gt;\n\nuser\n&lt;p&gt;Username to connect to database as.&lt;/p&gt;\n\npassword\n&lt;p&gt;Password for the user.&lt;/p&gt;\n\nssl\n&lt;p&gt;(default: false): Whether to use SSL or not.&lt;/p&gt;\n\nconnection_parameters\n&lt;p&gt;(default: []): List of 2-tuples, where key and value must be binary\nstrings. You can include any Postgres connection parameter here, such as\n&lt;code&gt;#(&quot;application_name&quot;, &quot;myappname&quot;)&lt;/code&gt; and &lt;code&gt;#(&quot;timezone&quot;, &quot;GMT&quot;)&lt;/code&gt;.&lt;/p&gt;\n\npool_size\n&lt;p&gt;(default: 1): Number of connections to keep open with the database&lt;/p&gt;\n\nqueue_target\n&lt;p&gt;(default: 50) Checking out connections is handled through a queue. If it\ntakes longer than queue_target to get out of the queue for longer than\nqueue_interval then the queue_target will be doubled and checkouts will\nstart to be dropped if that target is surpassed.&lt;/p&gt;\n\nqueue_interval\n&lt;p&gt;(default: 1000)&lt;/p&gt;\n\nidle_interval\n&lt;p&gt;(default: 1000): The database is pinged every idle_interval when the\nconnection is idle.&lt;/p&gt;\n\ntrace\n&lt;p&gt;trace (default: false): pgo is instrumented with &lt;a href=&quot;https://opencensus.io/&quot;&gt;OpenCensus&lt;/a&gt; and\nwhen this option is true a span will be created (if sampled).&lt;/p&gt;\n\nip_version\n&lt;p&gt;Which internet protocol to use for this connection&lt;/p&gt;\n\nSynonyms:\ngleam/pgo.Config\ngleam/pgo Config","url":"gleam/pgo.html#Config"},{"doc":"gleam/pgo","title":"Connection","content":"pub type Connection\n A pool of one or more database connections against which queries can be\n made.\n\n Created using the `connect` function and shut-down with the `disconnect`\n function.\n\nSynonyms:\ngleam/pgo.Connection\ngleam/pgo Connection","url":"gleam/pgo.html#Connection"},{"doc":"gleam/pgo","title":"IpVersion","content":"pub type IpVersion {\n  Ipv4\n  Ipv6\n}\n The internet protocol version to use.\nIpv4\n Internet Protocol version 6 (IPv6)\n\nIpv6\n Internet Protocol version 4 (IPv4)\n\nSynonyms:\ngleam/pgo.IpVersion\ngleam/pgo IpVersion","url":"gleam/pgo.html#IpVersion"},{"doc":"gleam/pgo","title":"QueryError","content":"pub type QueryError {\n  ConstraintViolated(\n    message: String,\n    constraint: String,\n    detail: String,\n  )\n  PostgresqlError(code: String, name: String, message: String)\n  UnexpectedArgumentCount(expected: Int, got: Int)\n  UnexpectedArgumentType(expected: String, got: String)\n  UnexpectedResultType(DecodeErrors)\n  ConnectionUnavailable\n}\n\nConstraintViolated(\n  message: String,\n  constraint: String,\n  detail: String,\n)\n The query failed as a database constraint would have been violated by the\n change.\n\nPostgresqlError(code: String, name: String, message: String)\n The query failed within the database.\n https://www.postgresql.org/docs/current/errcodes-appendix.html\n\nUnexpectedArgumentCount(expected: Int, got: Int)\n\n\nUnexpectedArgumentType(expected: String, got: String)\n One of the arguments supplied was not of the type that the query required.\n\nUnexpectedResultType(DecodeErrors)\n The rows returned by the database could not be decoded using the supplied\n dynamic decoder.\n\nConnectionUnavailable\n No connection was available to execute the query. This may be due to\n invalid connection details such as an invalid username or password.\n\nSynonyms:\ngleam/pgo.QueryError\ngleam/pgo QueryError","url":"gleam/pgo.html#QueryError"},{"doc":"gleam/pgo","title":"Returned","content":"pub type Returned(t) {\n  Returned(count: Int, rows: List(t))\n}\n The rows and number of rows that are returned by a database query.\nReturned(count: Int, rows: List(t))\n\n\nSynonyms:\ngleam/pgo.Returned\ngleam/pgo Returned","url":"gleam/pgo.html#Returned"},{"doc":"gleam/pgo","title":"Value","content":"pub type Value\n A value that can be sent to PostgreSQL as one of the arguments to a\n parameterised SQL query.\n\nSynonyms:\ngleam/pgo.Value\ngleam/pgo Value","url":"gleam/pgo.html#Value"},{"doc":"gleam/pgo","title":"array","content":"pub fn array(a: List(a)) -&gt; Value\n Coerce a timestamp represented as `#(#(year, month, day), #(hour, minute, second))` into a `Value`.\nSynonyms:\ngleam/pgo.array\ngleam/pgo array","url":"gleam/pgo.html#array"},{"doc":"gleam/pgo","title":"bool","content":"pub fn bool(a: Bool) -&gt; Value\n\nSynonyms:\ngleam/pgo.bool\ngleam/pgo bool","url":"gleam/pgo.html#bool"},{"doc":"gleam/pgo","title":"bytea","content":"pub fn bytea(a: BitArray) -&gt; Value\n\nSynonyms:\ngleam/pgo.bytea\ngleam/pgo bytea","url":"gleam/pgo.html#bytea"},{"doc":"gleam/pgo","title":"connect","content":"pub fn connect(a: Config) -&gt; Connection\n Start a database connection pool.\n\n The pool is started in a new process and will asynchronously connect to the\n PostgreSQL instance specified in the config. If the configuration is invalid\n or it cannot connect for another reason it will continue to attempt to\n connect, and any queries made using the connection pool will fail.\nSynonyms:\ngleam/pgo.connect\ngleam/pgo connect","url":"gleam/pgo.html#connect"},{"doc":"gleam/pgo","title":"decode_timestamp","content":"pub fn decode_timestamp(\n  value: Dynamic,\n) -&gt; Result(\n  #(#(Int, Int, Int), #(Int, Int, Int)),\n  List(DecodeError),\n)\n Checks to see if the value is formatted as `#(#(Int, Int, Int), #(Int, Int, Int))`\n to represent `#(#(year, month, day), #(hour, minute, second))`, and returns the\n value if it is.\nSynonyms:\ngleam/pgo.decode_timestamp\ngleam/pgo decode_timestamp","url":"gleam/pgo.html#decode_timestamp"},{"doc":"gleam/pgo","title":"default_config","content":"pub fn default_config() -&gt; Config\n The default configuration for a connection pool, with a single connection.\n You will likely want to increase the size of the pool for your application.\n\nSynonyms:\ngleam/pgo.default_config\ngleam/pgo default_config","url":"gleam/pgo.html#default_config"},{"doc":"gleam/pgo","title":"disconnect","content":"pub fn disconnect(a: Connection) -&gt; Nil\n Shut down a connection pool.\nSynonyms:\ngleam/pgo.disconnect\ngleam/pgo disconnect","url":"gleam/pgo.html#disconnect"},{"doc":"gleam/pgo","title":"error_code_name","content":"pub fn error_code_name(error_code: String) -&gt; Result(String, Nil)\n Get the name for a PostgreSQL error code.\n\n \n &gt; error_code_name(&quot;01007&quot;)\n Ok(&quot;privilege_not_granted&quot;)\n \n\n https://www.postgresql.org/docs/current/errcodes-appendix.html\nSynonyms:\ngleam/pgo.error_code_name\ngleam/pgo error_code_name","url":"gleam/pgo.html#error_code_name"},{"doc":"gleam/pgo","title":"execute","content":"pub fn execute(\n  query sql: String,\n  on pool: Connection,\n  with arguments: List(Value),\n  expecting decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(Returned(a), QueryError)\n Run a query against a PostgreSQL database.\n\n The provided dynamic decoder is used to decode the rows returned by\n PostgreSQL. If you are not interested in any returned rows you may want to\n use the `dynamic.dynamic` decoder.\n\nSynonyms:\ngleam/pgo.execute\ngleam/pgo execute","url":"gleam/pgo.html#execute"},{"doc":"gleam/pgo","title":"float","content":"pub fn float(a: Float) -&gt; Value\n\nSynonyms:\ngleam/pgo.float\ngleam/pgo float","url":"gleam/pgo.html#float"},{"doc":"gleam/pgo","title":"int","content":"pub fn int(a: Int) -&gt; Value\n\nSynonyms:\ngleam/pgo.int\ngleam/pgo int","url":"gleam/pgo.html#int"},{"doc":"gleam/pgo","title":"null","content":"pub fn null() -&gt; Value\n\nSynonyms:\ngleam/pgo.null\ngleam/pgo null","url":"gleam/pgo.html#null"},{"doc":"gleam/pgo","title":"nullable","content":"pub fn nullable(\n  inner_type: fn(a) -&gt; Value,\n  value: Option(a),\n) -&gt; Value\n\nSynonyms:\ngleam/pgo.nullable\ngleam/pgo nullable","url":"gleam/pgo.html#nullable"},{"doc":"gleam/pgo","title":"text","content":"pub fn text(a: String) -&gt; Value\n\nSynonyms:\ngleam/pgo.text\ngleam/pgo text","url":"gleam/pgo.html#text"},{"doc":"gleam/pgo","title":"timestamp","content":"pub fn timestamp(\n  a: #(#(Int, Int, Int), #(Int, Int, Int)),\n) -&gt; Value\n\nSynonyms:\ngleam/pgo.timestamp\ngleam/pgo timestamp","url":"gleam/pgo.html#timestamp"},{"doc":"gleam/pgo","title":"url_config","content":"pub fn url_config(database_url: String) -&gt; Result(Config, Nil)\n Parse a database url into configuration that can be used to start a pool.\nSynonyms:\ngleam/pgo.url_config\ngleam/pgo url_config","url":"gleam/pgo.html#url_config"},{"doc":"gleam/pgo","title":"gleam/pgo","content":" Postgresql client\n\n Gleam wrapper around pgo library","url":"gleam/pgo.html"}]);